# Generated by Django 5.0.3 on 2024-03-12 07:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoauthtoken', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='token',
            name='refresh_token',
            field=models.CharField(max_length=511),
        ),
        migrations.AlterField(
            model_name='token',
            name='token',
            field=models.CharField(max_length=511),
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.contrib import auth
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from django.conf import settings

from djangoauthtoken.models import TokenUser, Token
from djangoauthtoken.utils import get_or_create_csrf_token


@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
@csrf_exempt
def login(request):
    """
    Login view
    It expects username and password.
    #TODO: Flag to switch to Email.
    """
    data = request.data
    password = data['password']
    try:
        if settings.USERNAME_LOGIN_METHOD:
            _user = data['username']
            _auth = auth.authenticate(username=_user, password=password)
        else:
            _user = data['email']
            _auth = auth.authenticate(username=_user, password=password)
        if _auth:
            user = TokenUser.objects.get(username=_user) if settings.USERNAME_LOGIN_METHOD else TokenUser.objects.get(
                email=_user)
            user_token = Token(user=user)
            user_token.save()
            _csrf = get_or_create_csrf_token(request)

            return Response({
                "id": user.id,
                "user": user.username if settings.USERNAME_LOGIN_METHOD else user.email,
                "token": user_token.token,
                "refresh_token": user_token.refresh_token,
                "expires_at": user_token.expiry_time
            },
                status=status.HTTP_200_OK,
                headers={
                    'X-CSRFToken': _csrf
                })
        else:
            # Create your own Exception layer.
            raise ObjectDoesNotExist
    except ObjectDoesNotExist:
        return Response(status=status.HTTP_404_NOT_FOUND)



from django.db import models

class Base(models.Model):
    """
    """

    MAX_LENGTH_SMALL = 127
    MAX_LENGTH_MEDIUM = 255
    MAX_LENGTH_LARGE = 511

    created_at = models.DateTimeField(auto_now_add=True, null=False)
    updated_at = models.DateTimeField(auto_now=True, null=False)

    class Meta:
        abstract = True



from django.contrib.auth.models import AbstractUser
import pytz
import datetime

from django.db import models
from django.core.validators import RegexValidator
from django.conf import settings

from djangoauthtoken.models import Base
from djangoauthtoken.models.custom_fields import CaseInsensitiveEmailField


tz = [(item, datetime.datetime.now(pytz.timezone(item)).strftime("%z") + " " + item) for item in pytz.all_timezones]

phone_regex = RegexValidator(
    regex=r'^\+?1?\d{9,15}',
    message="Phone numbers must be in International format: '+14169058762'. Up to 15 digits "
)


class TokenUser(AbstractUser, Base):

    phone_number = models.CharField(validators=[phone_regex], max_length=17, null=True)
    if not settings.USERNAME_LOGIN_METHOD:
        email = CaseInsensitiveEmailField(max_length=Base.MAX_LENGTH_LARGE, unique=True)
        username = CaseInsensitiveEmailField(max_length=Base.MAX_LENGTH_LARGE, blank=True, required=False)
        REQUIRED_FIELDS = ["email"]
    
    def __str__(self) -> str:
        return self.username if settings.USERNAME_LOGIN_METHOD else self.email



from datetime import datetime

def get_epoch(dt=None) -> int:
    """
    :param dt: A datetime aware object
    :return: An int object representing time in epoch in utc
    """
    if not dt:
        dt = datetime.utcnow()
    return int(dt.timestamp() * 10000)




from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.response import Response

from django.conf import settings
from djangoauthtoken.models import TokenUser, Token
from djangoauthtoken.utils import get_or_create_csrf_token

@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
def sign_up(request):
    """
    Sign up request
    """
    try:
        data = request.data
        if settings.USERNAME_LOGIN_METHOD:
            username = data['username']
            if 'email' in data.keys():
                email = data['email']
                user, exists = TokenUser.objects.get_or_create(
                    username=username,
                    email=email
                )
            else:
                user, exists = TokenUser.objects.get_or_create(
                    username=username
                )
        else:
            email = data['email']
            if 'username' in data.keys():
                username = data['username']
                user, exists = TokenUser.objects.get_or_create(
                    username=username,
                    email=email
                )
            else:
                user, exists = TokenUser.objects.get_or_create(
                    email=email
                )
        password = data['password']
        user.set_password(password)
        user.save()

        if not exists:
            raise Exception

        user_token = Token(user=user)
        user_token.save()
        _csrf = get_or_create_csrf_token(request)

        return Response({
            "user": user.username if settings.USERNAME_LOGIN_METHOD else user.email,
            "id": user.id,
            "token": user_token.token,
            "refresh_token": user_token.refresh_token,
            "expires_at": user_token.expiry_time
        }, status=status.HTTP_201_CREATED)
    except:
        return Response(status=status.HTTP_400_BAD_REQUEST)




# Generated by Django 5.0.3 on 2024-04-03 06:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoauthtoken', '0002_alter_token_refresh_token_alter_token_token'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tokenuser',
            name='email',
            field=models.EmailField(blank=True, max_length=254, verbose_name='email address'),
        ),
    ]



import jwt

from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.response import Response

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from djangoauthtoken.models import TokenUser, Token


@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
def refresh_token(request):
    """
    Refresh Token request.
    """
    try:
        data = request.data
        refresh_token = data['refresh_token']
        refresh_token_decode = jwt.decode(refresh_token,
                                          settings.JWT_SECRET,
                                          algorithms=settings.JWT_ALGO)
        # Check for user.
        user = TokenUser.objects.get(id=refresh_token_decode['user_id'])
        # check for token.
        token_user = Token.objects.get(user=user, refresh_token=refresh_token)
        if not token_user:
            raise ObjectDoesNotExist
        # save new token.
        token = Token(user=user)
        token.save()

        return Response({
            "id": user.id,
            "user": token_user.username if settings.USERNAME_LOGIN_METHOD else token_user.email,
            "token": token.token,
            "refresh_token": token.refresh_token,
            "expires_at": token.expiry_time
        },
            status=status.HTTP_200_OK)
    except:
        return Response(status=status.HTTP_400_BAD_REQUEST)



# Generated by Django 5.0.3 on 2024-03-12 07:37

import django.contrib.auth.models
import django.contrib.auth.validators
import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import djangoauthtoken.models.custom_fields.case_insensitive_email_field
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='TokenUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('email', djangoauthtoken.models.custom_fields.case_insensitive_email_field.CaseInsensitiveEmailField(max_length=511, unique=True)),
                ('phone_number', models.CharField(max_length=17, null=True, validators=[django.core.validators.RegexValidator(message="Phone numbers must be in International format: '+14169058762'. Up to 15 digits ", regex='^\\+?1?\\d{9,15}')])),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Token',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('token', models.CharField(max_length=127)),
                ('refresh_token', models.CharField(max_length=127)),
                ('expiry_time', models.IntegerField(default=600)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""djangoauthtoken URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from rest_framework import routers

from djangoauthtoken.views import (
    UserViewSet, 
    login,
    refresh_token,
    sign_up
)


router = routers.DefaultRouter()
router.register(r'fetchusers', UserViewSet)

urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include(router.urls)),
    path('login', login, name='login'),
    path('refresh_token', refresh_token, name='refresh_token'),
    path('sign_up', sign_up, name='sing_up'),
]
